%% Estimayion of Confidence Limits using Multiple realizations, Bootstrap and Monte carlo on simulated realistic HRV process
%%% Generates FIG. 3 of Beda, Simpson & Faes 2016
clear;close all;clc;

fc=1; % simulated sampling freq.
nfft=512; % number of points for spectral functions
N=300; % realization length

do_resampling='y'; % numrep resampling iterations on one realization of the simulation
do_bootstrap='y'; % numrep bootstrap iterations on one realization of the simulation
do_manysimu='n'; % numrep realizations of the simulation
numrep=1000; % number of repetitions of the analyses above

lo=25; hi=75; % percentile limits for confidence intervals
ySPmax=80; %y limts for plotting spectra

hfig=figure('Name','FIG. 3');

%% simulation of AR5 process, exact indexes
M=1; 
ro0=0.65; %fr0=0
ro1=0.8; fr1=0.1;
ro2=0.92; fr2=0.25;
Su=1;
        
z0=ro0;
z1a=ro1*(cos(2*pi*fr1)+1i*sin(2*pi*fr1)); z1b=ro1*(cos(2*pi*fr1)-1i*sin(2*pi*fr1));
z2a=ro2*(cos(2*pi*fr2)+1i*sin(2*pi*fr2)); z2b=ro2*(cos(2*pi*fr2)-1i*sin(2*pi*fr2));

z=[z0; z1a; z1b; z2a; z2b];
Apol=poly(z);
Am=-Apol(2:length(Apol));
p=length(Am);

%%% SPECTRAL DECOMPOSITION (using function from asd toolbox)
[Sk,Sm,freq,pot,potn]=asd_ARMASpectDec(Apol,1,Su,fc,nfft);
potY=sum(pot);
[freq,ifreq]=sort(freq);
pot=pot(ifreq);
f = (0:nfft-1)*(fc/(2*nfft)); f=f'; %frequency axis
Plfhf=pot(2)./pot(3);

%%% INFORMATION DECOMPOSITION (using function from its toolbox)
% numlags=5;
ret = its_CElinVAR1(Am,Su,p);
varY=ret.Sy; %verify variance
VarU=ret.Sy_y; % has to be equal to Su
Hy=ret.Hy;
Hy_y=ret.Hy_y;
Sy=Hy-Hy_y; % storage


%% realization, estimated indexes
U = sqrt(Su)*randn(N,1);
Y=U;
for n=2:N 
    k=max(n-p,1);
    Y(n) = Am(1:n-k) * Y(n-1:-1:k) + U(n);
end
Y=Y-mean(Y);

% identification (use exact model order)
[eAm,eSu,Yp,Up,covBS]=idAR(Y,p,'n'); 
varUp=var(Up);

%%% spectral decomposition
eApol=[1 -eAm'];
[eSk,eSm,efreq,epot,epotn]=asd_ARMASpectDec(eApol,1,eSu,fc,nfft);
epotY=sum(epot);
[efreq,iefreq]=sort(efreq);
epot=epot(iefreq);
ePlfhf=epot(2)./epot(3);

%%% info decomposition
ret = its_CElinVAR1(eAm',eSu,p);
epotY_ver=ret.Sy; %verify variance
eVarU=ret.Sy_y; % has to be equal to Su
eHy=ret.Hy;
eHy_y=ret.Hy_y;
eSy=eHy-eHy_y;%storage


%% Confidence Limits: Many Realizations
if do_manysimu=='y'
    disp('many realizations...');
    nn=NaN*ones(1,numrep); mepotY=nn; mepotY_ver=nn; meVarU=nn; meHy=nn; meHy_y=nn; meSy=nn;
    mefreq=NaN*ones(length(freq),numrep); mepot=mefreq;
    for im=1:numrep
        %generate realization
        Ur = sqrt(Su)*randn(N,1);
        Yr=Ur;
        for n=2:N 
            k=max(n-p,1);
            Yr(n) = Am(1:n-k) * Yr(n-1:-1:k) + Ur(n);
        end
        Yr=Yr-mean(Yr);
        
        % identification (use exact model order)
        [meAm,meSu]=idAR(Yr,p,'y');     
        %%% spectral dec.
        meApol=[1 -meAm'];
        [Sktmp,Smtmp,freqtmp,pottmp,potntmp]=asd_ARMASpectDec(meApol,1,meSu,fc,nfft);
        meSm(:,im)=Smtmp;
        meSk(:,:,im)=Sktmp;
        [freqtmp,iefreqtmp]=sort(freqtmp);
        pottmp=pottmp(iefreqtmp);
        mepotY(im)=sum(pottmp);
        mefreq(:,im)=freqtmp;
        mepot(:,im)=pottmp;
        %%% info dec
        ret = its_CElinVAR1(meAm',meSu,p);
        mepotY_ver(im)=ret.Sy;
        meVarU(im)=ret.Sy_y;
        meHy(im)=ret.Hy;
        meHy_y(im)=ret.Hy_y;
        meSy(im)=ret.Hy-ret.Hy_y;
    end
    mePlfhf=mepot(2,:)./mepot(3,:);
    mD_Plfhf=prctile(mePlfhf,[lo 50 hi]);
    mM_Plfhf=mD_Plfhf(2); %median
    mL_Plfhf=mD_Plfhf(2)-mD_Plfhf(1); %lo-perc
    mU_Plfhf=mD_Plfhf(3)-mD_Plfhf(2); %hi-perc
        
    % collect results
    mD_potY=prctile(mepotY,[lo 50 hi]);
    mM_potY=mD_potY(2); %median
    mL_potY=mD_potY(2)-mD_potY(1); %lo-perc
    mU_potY=mD_potY(3)-mD_potY(2); %hi-perc

    % spectral
    mD_pot=prctile(mepot,[lo 50 hi],2);
    mM_pot=mD_pot(:,2); %median
    mL_pot=mD_pot(:,2)-mD_pot(:,1); %lo-perc
    mU_pot=mD_pot(:,3)-mD_pot(:,2); %hi-perc

    mD_freq=prctile(mefreq,[lo 50 hi],2);
    mM_freq=mD_freq(:,2); %median
    mL_freq=mD_freq(:,2)-mD_freq(:,1); %lo-perc
    mU_freq=mD_freq(:,3)-mD_freq(:,2); %hi-perc

    % information
    mD_Hy=prctile(meHy,[lo 50 hi]);
    mM_Hy=mD_Hy(2); %median
    mL_Hy=mD_Hy(2)-mD_Hy(1); %lo-perc
    mU_Hy=mD_Hy(3)-mD_Hy(2); %hi-perc

    mD_Hy_y=prctile(meHy_y,[lo 50 hi]);
    mM_Hy_y=mD_Hy_y(2); %median
    mL_Hy_y=mD_Hy_y(2)-mD_Hy_y(1); %lo-perc
    mU_Hy_y=mD_Hy_y(3)-mD_Hy_y(2); %hi-perc

    mD_Sy=prctile(meSy,[lo 50 hi]);
    mM_Sy=mD_Sy(2); %median
    mL_Sy=mD_Sy(2)-mD_Sy(1); %lo-perc
    mU_Sy=mD_Sy(3)-mD_Sy(2); %hi-perc

    %%% figura rappresentativa
    mD_Sm=prctile(meSm,[lo 50 hi],2);
%     figure(11);
    figure(hfig); subplot(2,9,[10 11 12]);
    plot(f,Sm,'color',[0.5 0.5 0.5]);hold on;
    plot(f,mD_Sm,'k:');
    hold off;
    axis([0 fc/2 0 1.05*max(mD_Sm(:,3))]);title('(d) many reps conf.limits');
    ylim([0 ySPmax]);

end

%% Confidence Limits: Bootstrap (BS)
if do_bootstrap=='y'
    % perform bootstrap
    Yi=Y(1:p);
    disp('bootstrapping...');
    ticID_boot=tic;
    out_boot=bootstrp(numrep,@(x)idARboot(x,eAm,Yi),Up);
    beAm=out_boot(:,1:p);
    beSu=out_boot(:,p+1);
    timeBS=toc(ticID_boot);
    
    %get indexes
    disp('collecting results...');
    nn=NaN*ones(1,numrep); bepotY=nn; bepotY_ver=nn; beVarU=nn; beHy=nn; beHy_y=nn; beSy=nn;
    befreq=NaN*ones(length(freq),numrep); bepot=befreq;
    for ib=1:numrep
        Apol_b=[1 -beAm(ib,:)];
        [Sktmp,Smtmp,freqtmp,pottmp,potntmp]=asd_ARMASpectDec(Apol_b,1,beSu(ib),fc,nfft);
        beSm(:,ib)=Smtmp;
        beSk(:,:,ib)=Sktmp;
        [freqtmp,iefreqtmp]=sort(freqtmp);
        pottmp=pottmp(iefreqtmp);

        bepotY(ib)=sum(pottmp);
        befreq(:,ib)=freqtmp;
        bepot(:,ib)=pottmp;
        
        ret = its_CElinVAR1(beAm(ib,:),beSu(ib),p);
        bepotY_ver(ib)=ret.Sy;
        beVarU(ib)=ret.Sy_y;
        beHy(ib)=ret.Hy;
        beHy_y(ib)=ret.Hy_y;
        beSy(ib)=ret.Hy-ret.Hy_y;
    end
    
    % collect results
    bD_potY=prctile(bepotY,[lo 50 hi]);
    bM_potY=bD_potY(2); %median
    bL_potY=bD_potY(2)-bD_potY(1); %lo-perc
    bU_potY=bD_potY(3)-bD_potY(2); %hi-perc
    
    % spectral
    bD_pot=prctile(bepot,[lo 50 hi],2);
    bM_pot=bD_pot(:,2); %median
    bL_pot=bD_pot(:,2)-bD_pot(:,1); %lo-perc
    bU_pot=bD_pot(:,3)-bD_pot(:,2); %hi-perc

    bD_freq=prctile(befreq,[lo 50 hi],2);
    bM_freq=bD_freq(:,2); %median
    bL_freq=bD_freq(:,2)-bD_freq(:,1); %lo-perc
    bU_freq=bD_freq(:,3)-bD_freq(:,2); %hi-perc

    % information
    bD_Hy=prctile(beHy,[lo 50 hi]);
    bM_Hy=bD_Hy(2); %median
    bL_Hy=bD_Hy(2)-bD_Hy(1); %lo-perc
    bU_Hy=bD_Hy(3)-bD_Hy(2); %hi-perc

    bD_Hy_y=prctile(beHy_y,[lo 50 hi]);
    bM_Hy_y=bD_Hy_y(2); %median
    bL_Hy_y=bD_Hy_y(2)-bD_Hy_y(1); %lo-perc
    bU_Hy_y=bD_Hy_y(3)-bD_Hy_y(2); %hi-perc

    bD_Sy=prctile(beSy,[lo 50 hi]);
    bM_Sy=bD_Sy(2); %median
    bL_Sy=bD_Sy(2)-bD_Sy(1); %lo-perc
    bU_Sy=bD_Sy(3)-bD_Sy(2); %hi-perc
    
    %%% figura rappresentativa
    bD_Sm=prctile(beSm,[lo 50 hi],2);
    %figure(12);
    figure(hfig); subplot(2,9,[7 8 9]);
    plot(f,Sm,'color',[0.5 0.5 0.5]);hold on;
    plot(f,eSm,'b'); plot(f,bD_Sm,'b:');
    hold off;
    axis([0 fc/2 0 1.05*max(bD_Sm(:,3))]);title('(c) BS conf.limits');
    ylim([0 ySPmax]);
    
    bePlfhf=bepot(2,:)./bepot(3,:);
    bD_Plfhf=prctile(bePlfhf,[lo 50 hi]);
    bM_Plfhf=bD_Plfhf(2); %median
    bL_Plfhf=bD_Plfhf(2)-bD_Plfhf(1); %lo-perc
    bU_Plfhf=bD_Plfhf(3)-bD_Plfhf(2); %hi-perc
    
    disp(['time BS: ' num2str(timeBS)]);

end


%% Confidence Limits: resampling through Monte Carlo (MC)
if do_resampling=='y'
    % perform resampling
    disp('resampling...');
    ticID_res=tic;
    out_res=idARres(numrep,eAm,eSu,covBS);
    reAm=out_res(:,1:p);
    reSu=out_res(:,p+1);
    timeMC=toc(ticID_res);
    
    %get indexes
    disp('collecting results...');
    nn=NaN*ones(1,numrep); repotY=nn; repotY_ver=nn; reVarU=nn; reHy=nn; reHy_y=nn; reSy=nn;
    refreq=NaN*ones(length(freq),numrep); repot=refreq;
    for ib=1:numrep
        
        %%% first check stability [from Nolte]
        E=eye(M*p);AA=[reAm(ib,:);E(1:end-M,:)];
        lambda=eig(AA); lambdamax=max(abs(lambda));
        if lambdamax>=1
            warning(['Resampling iteration ' int2str(ib) ' got unstable AR coeffs' ]);
        else
        Apol_b=[1 -reAm(ib,:)];
        [Sktmp,Smtmp,freqtmp,pottmp,potntmp]=asd_ARMASpectDec(Apol_b,1,reSu(ib),fc,nfft);        
        reSm(:,ib)=Smtmp;
        reSk(:,:,ib)=Sktmp;
        [freqtmp,iefreqtmp]=sort(freqtmp);
        pottmp=pottmp(iefreqtmp);

        repotY(ib)=sum(pottmp);
        refreq(:,ib)=freqtmp;
        repot(:,ib)=pottmp;
        
        ret = its_CElinVAR1(reAm(ib,:),reSu(ib),p);
        repotY_ver(ib)=ret.Sy;
        reVarU(ib)=ret.Sy_y;
        reHy(ib)=ret.Hy;
        reHy_y(ib)=ret.Hy_y;
        reSy(ib)=ret.Hy-ret.Hy_y;
        end
    end
    
    % collect results
    rD_potY=prctile(repotY,[lo 50 hi]);
    rM_potY=rD_potY(2); %median
    rL_potY=rD_potY(2)-rD_potY(1); %lo-perc
    rU_potY=rD_potY(3)-rD_potY(2); %hi-perc


    % spectral
    rD_pot=prctile(repot,[lo 50 hi],2);
    rM_pot=rD_pot(:,2); %median
    rL_pot=rD_pot(:,2)-rD_pot(:,1); %lo-perc
    rU_pot=rD_pot(:,3)-rD_pot(:,2); %hi-perc

    rD_freq=prctile(refreq,[lo 50 hi],2);
    rM_freq=rD_freq(:,2); %median
    rL_freq=rD_freq(:,2)-rD_freq(:,1); %lo-perc
    rU_freq=rD_freq(:,3)-rD_freq(:,2); %hi-perc

    % information
    rD_Hy=prctile(reHy,[lo 50 hi]);
    rM_Hy=rD_Hy(2); %median
    rL_Hy=rD_Hy(2)-rD_Hy(1); %lo-perc
    rU_Hy=rD_Hy(3)-rD_Hy(2); %hi-perc

    rD_Hy_y=prctile(reHy_y,[lo 50 hi]);
    rM_Hy_y=rD_Hy_y(2); %median
    rL_Hy_y=rD_Hy_y(2)-rD_Hy_y(1); %lo-perc
    rU_Hy_y=rD_Hy_y(3)-rD_Hy_y(2); %hi-perc

    rD_Sy=prctile(reSy,[lo 50 hi]);
    rM_Sy=rD_Sy(2); %median
    rL_Sy=rD_Sy(2)-rD_Sy(1); %lo-perc
    rU_Sy=rD_Sy(3)-rD_Sy(2); %hi-perc
    
    %%% figura rappresentativa
    rD_Sm=prctile(reSm,[lo 50 hi],2);
    %figure(13);
    figure(hfig); subplot(2,9,[4 5 6]);
    plot(f,Sm,'color',[0.5 0.5 0.5]);hold on;
    plot(f,eSm,'r'); plot(f,rD_Sm,'r:');
    hold off;
    axis([0 fc/2 0 1.05*max(rD_Sm(:,3))]);title('(b) MC conf.limits');
    ylim([0 ySPmax]);
    
    rePlfhf=repot(2,:)./repot(3,:);
    rD_Plfhf=prctile(rePlfhf,[lo 50 hi]);
    rM_Plfhf=rD_Plfhf(2); %median
    rL_Plfhf=rD_Plfhf(2)-rD_Plfhf(1); %lo-perc
    rU_Plfhf=rD_Plfhf(3)-rD_Plfhf(2); %hi-perc
    
    disp(['time MC: ' num2str(timeMC)]);
    
end


%% disp and plot
figure(hfig); subplot(2,9,[1 2 3]);
plot(f,Sm,'k');hold on;
for k=1:size(Sk,2)
    plot(f,Sk(:,k),'k:');
end
hold off;
axis([0 fc/2 0 1.1*max(Sm)]);title('(a) True Spectrum');

assetick=[1 2 3];
% x2=[1:4]'; dx=0.15;
assex={'BS'; 'gold st.'; 'MC'};
% figure(2);
subplot(2,9,[15 16]);
plot(2,Plfhf,'g.'); hold on; plot(1,ePlfhf,'gx'); plot(3,ePlfhf,'gx');
set(gca,'XTick',assetick); set(gca,'XTickLabel',assex);
title('(f) P_{LFHF}');
xlim([0 4]); ylim([2 5]);

% x3=[1:3]';
assex={'BS'; 'gold st.'; 'MC'};
% figure(3);
subplot(2,9,[13 14]);
plot(2,freq(2),'g.-'); hold on; plot(1,efreq(2),'gx'); plot(3,efreq(2),'gx');
set(gca,'XTick',assetick); set(gca,'XTickLabel',assex);
title('(e) f_{LF}');
xlim([0 4]); ylim([0.085 0.115]);

x4=[1:3]';
assex={'BS'; 'gold st.'; 'MC'};
% figure(4);
subplot(2,9,[17 18]);
plot(2,Sy,'g.'); hold on; plot(1,eSy,'gx'); plot(3,eSy,'gx');
set(gca,'XTick',assetick); set(gca,'XTickLabel',assex);
title('(g) S_X');
xlim([0 4]); ylim([0.95 1.1]);

if do_manysimu=='y'
    subplot(2,9,[15 16]); hold on; %figure(2); hold on;
    errorbar(2,mM_Plfhf,mL_Plfhf,mU_Plfhf,'k+');
    subplot(2,9,[13 14]); hold on; %figure(3); hold on;
    errorbar(2,mM_freq(2),mL_freq(2),mU_freq(2),'k+');
    subplot(2,9,[17 18]); hold on; %figure(4); hold on;
    errorbar(2,mM_Sy,mL_Sy,mU_Sy,'k+');
end
if do_bootstrap=='y'
    subplot(2,9,[15 16]); hold on; %figure(2); hold on;
    errorbar(1,bM_Plfhf,bL_Plfhf,bU_Plfhf,'b+');
    subplot(2,9,[13 14]); hold on; %figure(3); hold on;
    errorbar(1,bM_freq(2),bL_freq(2),bU_freq(2),'b+');
    subplot(2,9,[17 18]); hold on; %figure(4); hold on;
    errorbar(1,bM_Sy,bL_Sy,bU_Sy,'b+');
end
if do_resampling=='y'
    subplot(2,9,[15 16]); hold on; %figure(2); hold on;
    errorbar(3,rM_Plfhf,rL_Plfhf,rU_Plfhf,'r+');
    subplot(2,9,[13 14]); hold on; %figure(3); hold on;
    errorbar(3,rM_freq(2),rL_freq(2),rU_freq(2),'r+');
    subplot(2,9,[17 18]); hold on; %figure(4); hold on;
    errorbar(3,rM_Sy,rL_Sy,rU_Sy,'r+');
end



