%% IDENTIFICATION OF AR MODEL: Y(n)=a(1)Y(n-1)+...+A(p)Y(n-p)+U(n)
% makes use of mvregress (least squares solution)

%%% input:
% Y, Nx1 vector of time series
% p, model order
% veloce: flag for fast computation of id_AR (not computing variance/covariance of regression coeffs), default ='y'

%%% output:
% Am=[A(1)...A(p)], M*pM matrix of the estimated MVAR model coefficients
% Su, estimated input variance
% Yp, estimated time series
% Up, estimated residuals
% covBS= variance/covariance matrix of coefficients and residual covariance


function [Am,Su,Yp,Up,covBS]=idAR(Y,p,veloce)

if exist('veloce','var')==0
    veloce='y';
end

N=size(Y,1);

% IDENTIFICATION
Z=NaN*ones(N-p,p); % observation matrix
for j=1:p
    Z(:,j)=Y(p+1-j:N-j);
end
Yb=Y(p+1:N);

if veloce=='y'
    [Am, Su]=mvregress(Z,Yb);
else
    [Am, Su, Up, covBS]=mvregress(Z,Yb,'varformat','full');
end

Yp=Z*Am; 
Yp=[NaN*ones(p,1); Yp]; % predicted data


end


